@@ -3,7 +3,16 @@
 
 #include <sys/types.h>
 
-typedef enum { EMPTY, STRING, FLOAT, INT, OPERATOR, LIDENT, FIDENT, LOCATION, EEK } Type;
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+typedef enum {
+	EMPTY
+#ifndef __cplusplus
+	, STRING, FLOAT, INT, OPERATOR, LIDENT, FIDENT, LOCATION, EEK
+#endif
+} Type;
 
 typedef enum { PLUS, MINUS, MUL, DIV, OP, CP, COMMA, LT, LE, GE, GT, ISEQUAL, ABOUTEQ, NE, POW, MOD } Operator;
 
@@ -15,7 +24,7 @@ typedef struct
     char *string;
     double flt;
     long integer;
-    Operator operator;
+    Operator op;
     char *lident;
     int fident;
     int location[3];
@@ -25,6 +34,10 @@ typedef struct
 
 int identcode(const char *s, size_t len);
 Token **scan(const char **s);
-void print(char *s, size_t size, int star, int quote, int scientific, int precision, Token **n);
+void print(char *s, size_t size, size_t chars, int quote, int scientific, int precision, Token **n);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif
