@@ -10,7 +10,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "cat.h"
+
 #include "csv.h"
 #include "default.h"
 #include "display.h"
@@ -20,8 +20,9 @@
 #include "parser.h"
 #include "scanner.h"
 #include "sheet.h"
+#include "utf8.h"
 #include "xdr.h"
-#include "version.h"
+
 /*}}}*/
 /* #defines */ /*{{{*/
 #define SHEET(s,x,y,z) (*(s->sheet+(x)*s->dimz*s->dimy+(y)*s->dimz+(z)))
@@ -219,14 +220,14 @@ void resize(Sheet *sheet, int x, int y, int z)
     int *newcolumn;
     unsigned int ndimx,ndimy,ndimz;
     /*}}}*/
-  
+
     sheet->changed=1;
     ndimx=(x>=sheet->dimx ? x+1 : sheet->dimx);
     ndimy=(y>=sheet->dimy ? y+1 : sheet->dimy);
     ndimz=(z>=sheet->dimz ? z+1 : sheet->dimz);
     /* allocate new sheet */ /*{{{*/
     newsheet=malloc(ndimx*ndimy*ndimz*sizeof(Cell*));
-    for (x=0; x<ndimx; ++x) for (y=0; y<ndimy; ++y) for (z=0; z<ndimz; ++z)  
+    for (x=0; x<ndimx; ++x) for (y=0; y<ndimy; ++y) for (z=0; z<ndimz; ++z)
     {
       if (x<sheet->dimx && y<sheet->dimy && z<sheet->dimz) *(newsheet+x*ndimz*ndimy+y*ndimz+z)=SHEET(sheet,x,y,z);
       else *(newsheet+x*ndimz*ndimy+y*ndimz+z)=(Cell*)0;
@@ -270,6 +271,8 @@ void initcell(Sheet *sheet, int x, int y, int z)
     SHEET(sheet,x,y,z)->adjust=AUTOADJUST;
     SHEET(sheet,x,y,z)->precision=-1;
     SHEET(sheet,x,y,z)->shadowed=0;
+    SHEET(sheet,x,y,z)->bold=0;
+    SHEET(sheet,x,y,z)->underline=0;
     SHEET(sheet,x,y,z)->scientific=DEF_SCIENTIFIC;
     SHEET(sheet,x,y,z)->value.type=EMPTY;
     SHEET(sheet,x,y,z)->resvalue.type=EMPTY;
@@ -403,7 +406,7 @@ void freecell(Sheet *sheet, int x, int y, int z)
     tvecfree(SHEET(sheet,x,y,z)->ccontents);
     tfree(&(SHEET(sheet,x,y,z)->value));
     free(SHEET(sheet,x,y,z));
-    SHEET(sheet,x,y,z)=(Cell*)0;      
+    SHEET(sheet,x,y,z)=(Cell*)0;
     sheet->changed=1;
   }
 }
@@ -453,6 +456,7 @@ void putcont(Sheet *sheet, int x, int y, int z, Token **t, int c)
     tvecfree(SHEET(sheet,x,y,z)->contents);
     SHEET(sheet,x,y,z)->contents=t;
   }
+  redraw_cell(sheet, x, y, z);
 }
 /*}}}*/
 /* getcont       -- get contents */ /*{{{*/
@@ -477,7 +481,7 @@ Token getvalue(Sheet *sheet, int x, int y, int z)
   /* return error */ /*{{{*/
   {
     result.type=EEK;
-    result.u.err=mystrmalloc(NEGLOC);
+    result.u.err=mystrmalloc(_("Negative index"));
   }
   /*}}}*/
   else if (getcont(sheet,x,y,z,2)==(Token**)0)
@@ -554,9 +558,9 @@ void update(Sheet *sheet)
   do
   {
     sheet->clk=0;
-    if (iterating==1) 
+    if (iterating==1)
     {
-      line_msg((const char*)0,"Calculating running, press a key to abort it");
+      line_msg((const char*)0,_("Calculating running, press Escape to abort it"));
       ++iterating;
     }
     else if (iterating==0) ++iterating;
@@ -586,8 +590,9 @@ void update(Sheet *sheet)
     }
     upd_clock=0;
   } while (sheet->clk && !(kp=keypressed()));
-  if (iterating==2) line_msg((const char*)0,kp ? "Calculation aborted" : "Calculation finished");
+  if (iterating==2) line_msg((const char*)0,kp ? _("Calculation aborted") : _("Calculation finished"));
   sheet->clk=0;
+  redraw_sheet(sheet);
 }
 /*}}}*/
 /* geterror      -- get malloc()ed error string */ /*{{{*/
@@ -596,7 +601,7 @@ char *geterror(Sheet *sheet, int x, int y, int z)
   Token v;
 
   assert(sheet!=(Sheet*)0);
-  if ((v=getvalue(sheet,x,y,z)).type!=EEK) 
+  if ((v=getvalue(sheet,x,y,z)).type!=EEK)
   {
     tfree(&v);
     return (char*)0;
@@ -608,14 +613,14 @@ char *geterror(Sheet *sheet, int x, int y, int z)
 }
 /*}}}*/
 /* printvalue    -- get ASCII representation of value */ /*{{{*/
-void printvalue(char *s, size_t size, int star, int quote, int scientific, int precision, Sheet *sheet, int x, int y, int z)
+void printvalue(char *s, size_t size, size_t chars, int quote, int scientific, int precision, Sheet *sheet, int x, int y, int z)
 {
   Token *tv[2],t;
 
   assert(sheet!=(Sheet*)0);
   t=getvalue(sheet,x,y,z); tv[0]=&t;
   tv[1]=(Token*)0;
-  print(s,size,star,quote,scientific,precision,tv);
+  print(s,size,chars,quote,scientific,precision,tv);
   tfree(&t);
 }
 /*}}}*/
@@ -629,7 +634,7 @@ Adjust getadjust(Sheet *sheet, int x, int y, int z)
   }
   else if (SHEET(sheet,x,y,z)->adjust==AUTOADJUST) return (SHEET(sheet,x,y,z)->value.type==INT || SHEET(sheet,x,y,z)->value.type==FLOAT ? RIGHT : LEFT);
   else return (SHEET(sheet,x,y,z)->adjust);
-}  
+}
 /*}}}*/
 /* setadjust     -- set cell adjustment */ /*{{{*/
 void setadjust(Sheet *sheet, int x, int y, int z, Adjust adjust)
@@ -639,7 +644,7 @@ void setadjust(Sheet *sheet, int x, int y, int z, Adjust adjust)
   resize(sheet,x,y,z);
   initcell(sheet,x,y,z);
   SHEET(sheet,x,y,z)->adjust=adjust;
-}  
+}
 /*}}}*/
 
 /* shadow        -- shadow cell by left neighbour */ /*{{{*/
@@ -648,13 +653,41 @@ void shadow(Sheet *sheet, int x, int y, int z, int yep)
   sheet->changed=1;
   initcell(sheet,x,y,z);
   SHEET(sheet,x,y,z)->shadowed=yep;
-}  
+}
 /*}}}*/
 /* shadowed      -- is cell shadowed? */ /*{{{*/
 int shadowed(Sheet *sheet, int x, int y, int z)
 {
   return (SHADOWED(sheet,x,y,z));
-}  
+}
+/*}}}*/
+/* bold        -- bold font */ /*{{{*/
+void bold(Sheet *sheet, int x, int y, int z, int yep)
+{
+  sheet->changed=1;
+  initcell(sheet,x,y,z);
+  SHEET(sheet,x,y,z)->bold=yep;
+}
+/*}}}*/
+/* isbold      -- is cell bold? */ /*{{{*/
+int isbold(Sheet *sheet, int x, int y, int z)
+{
+  return (x<sheet->dimx && y<sheet->dimy && z<sheet->dimz && SHEET(sheet,x,y,z)!=(Cell*)0 && SHEET(sheet,x,y,z)->bold);
+}
+/*}}}*/
+/* underline        -- underline */ /*{{{*/
+void underline(Sheet *sheet, int x, int y, int z, int yep)
+{
+  sheet->changed=1;
+  initcell(sheet,x,y,z);
+  SHEET(sheet,x,y,z)->underline=yep;
+}
+/*}}}*/
+/* isunderline      -- is cell underlined? */ /*{{{*/
+int underlined(Sheet *sheet, int x, int y, int z)
+{
+  return (x<sheet->dimx && y<sheet->dimy && z<sheet->dimz && SHEET(sheet,x,y,z)!=(Cell*)0 && SHEET(sheet,x,y,z)->underline);
+}
 /*}}}*/
 /* lockcell      -- lock cell */ /*{{{*/
 void lockcell(Sheet *sheet, int x, int y, int z, int yep)
@@ -662,19 +695,19 @@ void lockcell(Sheet *sheet, int x, int y, int z, int yep)
   sheet->changed=1;
   initcell(sheet,x,y,z);
   SHEET(sheet,x,y,z)->locked=yep;
-}  
+}
 /*}}}*/
 /* locked        -- is cell locked? */ /*{{{*/
 int locked(Sheet *sheet, int x, int y, int z)
 {
   return (x<sheet->dimx && y<sheet->dimy && z<sheet->dimz && SHEET(sheet,x,y,z)!=(Cell*)0 && SHEET(sheet,x,y,z)->locked);
-}  
+}
 /*}}}*/
 /* transparent   -- is cell transparent? */ /*{{{*/
 int transparent(Sheet *sheet, int x, int y, int z)
 {
   return (x<sheet->dimx && y<sheet->dimy && z<sheet->dimz && SHEET(sheet,x,y,z)!=(Cell*)0 && SHEET(sheet,x,y,z)->transparent);
-}  
+}
 /*}}}*/
 /* maketrans     -- make cell transparent */ /*{{{*/
 void maketrans(Sheet *sheet, int x, int y, int z, int yep)
@@ -682,7 +715,7 @@ void maketrans(Sheet *sheet, int x, int y, int z, int yep)
   sheet->changed=1;
   initcell(sheet,x,y,z);
   SHEET(sheet,x,y,z)->transparent=yep;
-}  
+}
 /*}}}*/
 /* igncell       -- ignore cell */ /*{{{*/
 void igncell(Sheet *sheet, int x, int y, int z, int yep)
@@ -690,13 +723,13 @@ void igncell(Sheet *sheet, int x, int y, int z, int yep)
   sheet->changed=1;
   initcell(sheet,x,y,z);
   SHEET(sheet,x,y,z)->ignored=yep;
-}  
+}
 /*}}}*/
 /* ignored       -- is cell ignored? */ /*{{{*/
 int ignored(Sheet *sheet, int x, int y, int z)
 {
   return (x<sheet->dimx && y<sheet->dimy && z<sheet->dimz && SHEET(sheet,x,y,z)!=(Cell*)0 && SHEET(sheet,x,y,z)->ignored);
-}  
+}
 /*}}}*/
 /* clk           -- clock cell */ /*{{{*/
 void clk(Sheet *sheet, int x, int y, int z)
@@ -710,7 +743,7 @@ void clk(Sheet *sheet, int x, int y, int z)
     SHEET(sheet,x,y,z)->clock_t2=1;
     sheet->clk=1;
   }
-}  
+}
 /*}}}*/
 /* setscientific -- cell value should be displayed in scientific notation */ /*{{{*/
 void setscientific(Sheet *sheet, int x, int y, int z, int yep)
@@ -719,14 +752,14 @@ void setscientific(Sheet *sheet, int x, int y, int z, int yep)
   resize(sheet,x,y,z);
   initcell(sheet,x,y,z);
   SHEET(sheet,x,y,z)->scientific=yep;
-}  
+}
 /*}}}*/
 /* getscientific -- should value be displayed in scientific notation? */ /*{{{*/
 int getscientific(Sheet *sheet, int x, int y, int z)
 {
   if (x<sheet->dimx && y<sheet->dimy && z<sheet->dimz && SHEET(sheet,x,y,z)!=(Cell*)0) return SHEET(sheet,x,y,z)->scientific;
   else return DEF_SCIENTIFIC;
-}  
+}
 /*}}}*/
 /* setprecision  -- set cell precision */ /*{{{*/
 void setprecision(Sheet *sheet, int x, int y, int z, int precision)
@@ -742,14 +775,14 @@ int getprecision(Sheet *sheet, int x, int y, int z)
 {
   if (x<sheet->dimx && y<sheet->dimy && z<sheet->dimz && SHEET(sheet,x,y,z)!=(Cell*)0) return (SHEET(sheet,x,y,z)->precision==-1 ? def_precision : SHEET(sheet,x,y,z)->precision);
   else return def_precision;
-}  
+}
 /*}}}*/
 /* getlabel      -- get cell label */ /*{{{*/
 const char *getlabel(Sheet *sheet, int x, int y, int z)
 {
   if (x>=sheet->dimx || y>=sheet->dimy || z>=sheet->dimz || SHEET(sheet,x,y,z)==(Cell*)0 || SHEET(sheet,x,y,z)->label==(char*)0) return "";
   else return (SHEET(sheet,x,y,z)->label);
-}  
+}
 /*}}}*/
 /* setlabel      -- set cell label */ /*{{{*/
 void setlabel(Sheet *sheet, int x, int y, int z, const char *buf, int update)
@@ -765,7 +798,7 @@ void setlabel(Sheet *sheet, int x, int y, int z, const char *buf, int update)
     cachelabels(sheet);
     forceupdate(sheet);
   }
-}  
+}
 /*}}}*/
 /* findlabel     -- return cell location for a given label */ /*{{{*/
 Token findlabel(Sheet *sheet, const char *label)
@@ -795,7 +828,7 @@ Token findlabel(Sheet *sheet, const char *label)
   else
   {
     result.type=EEK;
-    result.u.err=mystrmalloc(NOLABEL);
+    result.u.err=mystrmalloc(_("No such label"));
   }
   return result;
 }
@@ -849,7 +882,7 @@ const char *savexdr(Sheet *sheet, const char *name, unsigned int *count)
   XDR xdrs;
   int x,y,z;
   /*}}}*/
-  
+
   *count=0;
   if ((fp=fopen(name,"w"))==(FILE*)0) return strerror(errno);
   xdrstdio_create(&xdrs,fp,XDR_ENCODE);
@@ -904,15 +937,15 @@ const char *savetbl(Sheet *sheet, const char *name, int body, int x1, int y1, in
   int x,y,z;
   char buf[1024];
   char num[20];
-  char fullname[_POSIX_PATH_MAX];
+  char fullname[PATH_MAX];
   /*}}}*/
-  
+
   /* asserts */ /*{{{*/
   assert(sheet!=(Sheet*)0);
   assert(name!=(const char*)0);
   /*}}}*/
   *count=0;
-  for (z=z1; z<=z2; ++z) for (y=y1; y<=y2; ++y) if (shadowed(sheet,x1,y,z)) return NOSHADOW;
+  for (z=z1; z<=z2; ++z) for (y=y1; y<=y2; ++y) if (shadowed(sheet,x1,y,z)) return _("Shadowed cells in first column");
   if (!body && (fp=fopen(name,"w"))==(FILE*)0) return strerror(errno);
   for (z=z1; z<=z2; ++z)
   {
@@ -920,11 +953,11 @@ const char *savetbl(Sheet *sheet, const char *name, int body, int x1, int y1, in
     /* open new file */ /*{{{*/
     {
       sprintf(num,".%d",z);
-      fullname[_POSIX_PATH_MAX-strlen(num)-1]='\0';
-      (void)strncpy(fullname,name,_POSIX_PATH_MAX-strlen(num)-1);
+      fullname[sizeof(fullname)-strlen(num)-1]='\0';
+      (void)strncpy(fullname,name,sizeof(fullname)-strlen(num)-1);
       fullname[sizeof(fullname)-1]='\0';
       (void)strncat(fullname,num,sizeof(fullname)-strlen(num)-1);
-      fullname[sizeof(fullname)-1]='\0';  
+      fullname[sizeof(fullname)-1]='\0';
       if ((fp=fopen(fullname,"w"))==(FILE*)0) return strerror(errno);
     }
     /*}}}*/
@@ -940,6 +973,14 @@ const char *savetbl(Sheet *sheet, const char *name, int body, int x1, int y1, in
         {
           if (fputc_close('s',fp)==EOF) return strerror(errno);
         }
+        if (isbold(sheet,x,y,z))
+        {
+          if (fputc_close('b',fp)==EOF) return strerror(errno);
+        }
+        if (underlined(sheet,x,y,z))
+        {
+          if (fputc_close('u',fp)==EOF) return strerror(errno);
+        }
         else switch (getadjust(sheet,x,y,z))
         {
           case LEFT: if (fputc_close('l',fp)==EOF) return strerror(errno); break;
@@ -955,7 +996,7 @@ const char *savetbl(Sheet *sheet, const char *name, int body, int x1, int y1, in
       {
         if (!shadowed(sheet,x,y,z))
         {
-          if (x>x1 && fputc_close('\t',fp)==EOF) return strerror(errno);    
+          if (x>x1 && fputc_close('\t',fp)==EOF) return strerror(errno);
           if (SHEET(sheet,x,y,z)!=(Cell*)0)
           {
             char *bufp;
@@ -1026,30 +1067,30 @@ const char *savetext(Sheet *sheet, const char *name, int x1, int y1, int z1, int
   FILE *fp;
   int x,y,z;
   /*}}}*/
-  
+
   /* asserts */ /*{{{*/
   assert(sheet!=(Sheet*)0);
   assert(name!=(const char*)0);
   /*}}}*/
   *count=0;
-  for (z=z1; z<=z2; ++z) for (y=y1; y<=y2; ++y) if (shadowed(sheet,x1,y,z)) return NOSHADOW;
+  for (z=z1; z<=z2; ++z) for (y=y1; y<=y2; ++y) if (shadowed(sheet,x1,y,z)) return _("Shadowed cells in first column");
   if ((fp=fopen(name,"w"))==(FILE*)0) return strerror(errno);
   for (z=z1; z<=z2; ++z)
   {
-    for (y=y1; y<=y2; ++y)   
+    for (y=y1; y<=y2; ++y)
     {
       size_t size,fill;
-        
+
       for (x=x1; x<=x2; ++x)
       {
         size=cellwidth(sheet,x,y,z);
         if (SHEET(sheet,x,y,z)!=(Cell*)0)
         {
           char *buf;
-    
-          buf=malloc(size+1);
-          printvalue(buf,size+1,1,0,getscientific(sheet,x,y,z),getprecision(sheet,x,y,z),sheet,x,y,z);
-          adjust(getadjust(sheet,x,y,z),buf,size+1);
+
+          buf=malloc(size*UTF8SZ+1);
+          printvalue(buf,size*UTF8SZ+1,size,0,getscientific(sheet,x,y,z),getprecision(sheet,x,y,z),sheet,x,y,z);
+          adjust(getadjust(sheet,x,y,z),buf,size);
           if (fputs_close(buf,fp)==EOF)
           {
             free(buf);
@@ -1077,33 +1118,33 @@ const char *savetext(Sheet *sheet, const char *name, int x1, int y1, int z1, int
 }
 /*}}}*/
 /* savecsv       -- save as CSV */ /*{{{*/
-const char *savecsv(Sheet *sheet, const char *name, int x1, int y1, int z1, int x2, int y2, int z2, unsigned int *count)
+const char *savecsv(Sheet *sheet, const char *name, char sep, int x1, int y1, int z1, int x2, int y2, int z2, unsigned int *count)
 {
   /* variables */ /*{{{*/
   FILE *fp;
   int x,y,z;
   /*}}}*/
-  
+
   /* asserts */ /*{{{*/
   assert(sheet!=(Sheet*)0);
   assert(name!=(const char*)0);
   /*}}}*/
   *count=0;
-  for (z=z1; z<=z2; ++z) for (y=y1; y<=y2; ++y) if (shadowed(sheet,x1,y,z)) return NOSHADOW;
+  for (z=z1; z<=z2; ++z) for (y=y1; y<=y2; ++y) if (shadowed(sheet,x1,y,z)) return _("Shadowed cells in first column");
   if ((fp=fopen(name,"w"))==(FILE*)0) return strerror(errno);
   for (z=z1; z<=z2; ++z)
   {
-    for (y=y1; y<=y2; ++y)   
+    for (y=y1; y<=y2; ++y)
     {
       for (x=x1; x<=x2; ++x)
       {
-        if (x>x1) if (fputc_close('\t',fp)==EOF) return strerror(errno);
+        if (x>x1) if (fputc_close(sep,fp)==EOF) return strerror(errno);
         if (SHEET(sheet,x,y,z)!=(Cell*)0)
         {
           char *buf,*s;
-    
-          buf=malloc(255+1);
-          printvalue(buf,255+1,1,0,getscientific(sheet,x,y,z),getprecision(sheet,x,y,z),sheet,x,y,z);
+
+          buf=malloc(255*UTF8SZ+1);
+          printvalue(buf,255*UTF8SZ+1,255,0,getscientific(sheet,x,y,z),getprecision(sheet,x,y,z),sheet,x,y,z);
           if (SHEET(sheet,x,y,z)->value.type==STRING && fputc_close('"',fp)==EOF)
           {
             free(buf);
@@ -1147,9 +1188,9 @@ const char *saveport(Sheet *sheet, const char *name, unsigned int *count)
   fprintf(fp,"# This is a work sheet generated with teapot %s.\n",VERSION);
   for (z=sheet->dimz-1; z>=0; --z)
   {
-    for (y=sheet->dimy-1; y>=0; --y)   
+    for (y=sheet->dimy-1; y>=0; --y)
     {
-      for (x=sheet->dimx-1; x>=0; --x) 
+      for (x=sheet->dimx-1; x>=0; --x)
       {
         if (y==0) if (columnwidth(sheet,x,z)!=DEF_COLUMNWIDTH) fprintf(fp,"W%d %d %d\n",x,z,columnwidth(sheet,x,z));
         if (SHEET(sheet,x,y,z)!=(Cell*)0)
@@ -1159,13 +1200,15 @@ const char *saveport(Sheet *sheet, const char *name, unsigned int *count)
           if (SHEET(sheet,x,y,z)->label) fprintf(fp,"L%s ",SHEET(sheet,x,y,z)->label);
           if (SHEET(sheet,x,y,z)->precision!=-1) fprintf(fp,"P%d ",SHEET(sheet,x,y,z)->precision);
           if (SHEET(sheet,x,y,z)->shadowed) fprintf(fp,"S ");
+          if (SHEET(sheet,x,y,z)->bold) fprintf(fp,"B ");
+          if (SHEET(sheet,x,y,z)->underline) fprintf(fp,"U ");
           if (SHEET(sheet,x,y,z)->scientific!=DEF_SCIENTIFIC) fprintf(fp,"E ");
           if (SHEET(sheet,x,y,z)->locked) fprintf(fp,"C ");
           if (SHEET(sheet,x,y,z)->transparent) fprintf(fp,"T ");
           if (SHEET(sheet,x,y,z)->contents)
           {
             char buf[4096];
-          
+
             if (fputc_close(':',fp)==EOF) return strerror(errno);
             print(buf,sizeof(buf),0,1,SHEET(sheet,x,y,z)->scientific,SHEET(sheet,x,y,z)->precision,SHEET(sheet,x,y,z)->contents);
             if (fputs_close(buf,fp)==EOF) return strerror(errno);
@@ -1173,7 +1216,7 @@ const char *saveport(Sheet *sheet, const char *name, unsigned int *count)
           if (SHEET(sheet,x,y,z)->ccontents)
           {
             char buf[4096];
-          
+
             if (fputs_close("\\\n",fp)==EOF) return strerror(errno);
             print(buf,sizeof(buf),0,1,SHEET(sheet,x,y,z)->scientific,SHEET(sheet,x,y,z)->precision,SHEET(sheet,x,y,z)->ccontents);
             if (fputs_close(buf,fp)==EOF) return strerror(errno);
@@ -1203,6 +1246,8 @@ const char *loadport(Sheet *sheet, const char *name)
   char *label;
   Adjust adjust;
   int shadowed;
+  int bold;
+  int underline;
   int scientific;
   int locked;
   int transparent;
@@ -1235,6 +1280,8 @@ const char *loadport(Sheet *sheet, const char *name)
         contents=(Token**)0;
         ccontents=(Token**)0;
         shadowed=0;
+        bold=0;
+        underline=0;
         scientific=DEF_SCIENTIFIC;
         locked=0;
         transparent=0;
@@ -1244,7 +1291,7 @@ const char *loadport(Sheet *sheet, const char *name)
         x=posnumber(os,&ns);
         if (os==ns)
         {
-          sprintf(errbuf,XERR,line);
+          sprintf(errbuf,_("Parse error for x position in line %d"),line);
           err=errbuf;
           goto eek;
         }
@@ -1253,8 +1300,8 @@ const char *loadport(Sheet *sheet, const char *name)
         y=posnumber(os,&ns);
         if (os==ns)
         {
-          sprintf(errbuf,YERR,line);
-          err=errbuf;  
+          sprintf(errbuf,_("Parse error for y position in line %d"),line);
+          err=errbuf;
           goto eek;
         }
         while (*ns==' ') ++ns;
@@ -1262,8 +1309,8 @@ const char *loadport(Sheet *sheet, const char *name)
         z=posnumber(os,&ns);
         if (os==ns)
         {
-          sprintf(errbuf,ZERR,line);
-          err=errbuf;    
+          sprintf(errbuf,_("Parse error for z position in line %d"),line);
+          err=errbuf;
           goto eek;
         }
         /*}}}*/
@@ -1282,7 +1329,7 @@ const char *loadport(Sheet *sheet, const char *name)
                 case 'l': adjust=LEFT; ++ns; break;
                 case 'r': adjust=RIGHT; ++ns; break;
                 case 'c': adjust=CENTER; ++ns; break;
-                default:  sprintf(errbuf,ADJERR,line); err=errbuf; goto eek;
+                default:  sprintf(errbuf,_("Parse error for adjustment in line %d"),line); err=errbuf; goto eek;
               }
               break;
             }
@@ -1307,7 +1354,7 @@ const char *loadport(Sheet *sheet, const char *name)
               precision=posnumber(os,&ns);
               if (os==ns)
               {
-                sprintf(errbuf,PRECERR,line);
+                sprintf(errbuf,_("Parse error for precision in line %d"),line);
                 err=errbuf;
                 goto eek;
               }
@@ -1319,7 +1366,7 @@ const char *loadport(Sheet *sheet, const char *name)
             {
               if (x==0)
               {
-                sprintf(errbuf,SHADOWBOO2,x,y,z,line);
+                sprintf(errbuf,_("Trying to shadow cell (%d,%d,%d) in line %d"),x,y,z,line);
                 err=errbuf;
                 goto eek;
               }
@@ -1328,6 +1375,34 @@ const char *loadport(Sheet *sheet, const char *name)
               break;
             }
             /*}}}*/
+            /* U       -- underline */ /*{{{*/
+            case 'U':
+            {
+              if (x==0)
+              {
+                sprintf(errbuf,_("Trying to underline cell (%d,%d,%d) in line %d"),x,y,z,line);
+                err=errbuf;
+                goto eek;
+              }
+              ++ns;
+              underline=1;
+              break;
+            }
+            /*}}}*/
+            /* B       -- bold */ /*{{{*/
+            case 'B':
+            {
+              if (x==0)
+              {
+                sprintf(errbuf,_("Trying to bold cell (%d,%d,%d) in line %d"),x,y,z,line);
+                err=errbuf;
+                goto eek;
+              }
+              ++ns;
+              bold=1;
+              break;
+            }
+            /*}}}*/
             /* E       -- scientific */ /*{{{*/
             case 'E':
             {
@@ -1365,7 +1440,7 @@ const char *loadport(Sheet *sheet, const char *name)
             case '\0': break;
             /*}}}*/
             /* default -- error */ /*{{{*/
-            default: sprintf(errbuf,OPTERR,*ns,line); err=errbuf; goto eek;
+            default: sprintf(errbuf,_("Invalid option %c in line %d"),*ns,line); err=errbuf; goto eek;
             /*}}}*/
           }
         } while (*ns!=':' && *ns!='\0');
@@ -1378,7 +1453,7 @@ const char *loadport(Sheet *sheet, const char *name)
           if (contents==(Token**)0)
           {
             tvecfree(contents);
-            sprintf(errbuf,SYNERR,line);
+            sprintf(errbuf,_("Expression syntax error in line %d"),line);
             err=errbuf;
             goto eek;
           }
@@ -1397,7 +1472,7 @@ const char *loadport(Sheet *sheet, const char *name)
           if (ccontents==(Token**)0)
           {
             tvecfree(ccontents);
-            sprintf(errbuf,SYNERR,line);
+            sprintf(errbuf,_("Expression syntax error in line %d"),line);
             err=errbuf;
             goto eek;
           }
@@ -1408,7 +1483,9 @@ const char *loadport(Sheet *sheet, const char *name)
         SHEET(sheet,x,y,z)->label=label;
         SHEET(sheet,x,y,z)->precision=precision;
         SHEET(sheet,x,y,z)->shadowed=shadowed;
-        SHEET(sheet,x,y,z)->scientific=scientific;  
+        SHEET(sheet,x,y,z)->bold=bold;
+        SHEET(sheet,x,y,z)->underline=underline;
+        SHEET(sheet,x,y,z)->scientific=scientific;
         SHEET(sheet,x,y,z)->locked=locked;
         SHEET(sheet,x,y,z)->transparent=transparent;
         SHEET(sheet,x,y,z)->ignored=ignored;
@@ -1425,7 +1502,7 @@ const char *loadport(Sheet *sheet, const char *name)
         x=posnumber(os,&ns);
         if (os==ns)
         {
-          sprintf(errbuf,XERR,line);
+          sprintf(errbuf,_("Parse error for x position in line %d"),line);
           err=errbuf;
           goto eek;
         }
@@ -1434,7 +1511,7 @@ const char *loadport(Sheet *sheet, const char *name)
         z=posnumber(os,&ns);
         if (os==ns)
         {
-          sprintf(errbuf,ZERR,line);
+          sprintf(errbuf,_("Parse error for z position in line %d"),line);
           err=errbuf;
           goto eek;
         }
@@ -1445,10 +1522,10 @@ const char *loadport(Sheet *sheet, const char *name)
         width=posnumber(os,&ns);
         if (os==ns)
         {
-          sprintf(errbuf,WIDTHERR,line);
+          sprintf(errbuf,_("Parse error for width in line %d"),line);
           err=errbuf;
           goto eek;
-        }  
+        }
         /*}}}*/
         setwidth(sheet,x,z,width);
         break;
@@ -1460,7 +1537,7 @@ const char *loadport(Sheet *sheet, const char *name)
       /* default -- error */ /*{{{*/
       default:
       {
-        sprintf(errbuf,NOTAG,buf[0],line);
+        sprintf(errbuf,_("Unknown tag %c in line %d"),buf[0],line);
         err=errbuf;
         goto eek;
       }
@@ -1495,7 +1572,7 @@ const char *loadxdr(Sheet *sheet, const char *name)
 #if 0
     xdr_destroy(&xdrs);
     fclose(fp);
-    return WRONGXDRMAGIC;
+    return _("This is not a teapot worksheet in XDR format");
 #else
     xdr_destroy(&xdrs);
     rewind(fp);
@@ -1530,7 +1607,7 @@ const char *loadxdr(Sheet *sheet, const char *name)
         return strerror(olderror);
       }
       initcell(sheet,x,y,z);
-      if (xdr_cell(&xdrs,SHEET(sheet,x,y,z))==0) 
+      if (xdr_cell(&xdrs,SHEET(sheet,x,y,z))==0)
       {
         freecell(sheet,x,y,z);
         olderror=errno;
@@ -1549,7 +1626,7 @@ const char *loadxdr(Sheet *sheet, const char *name)
       sheet->changed=0;
       cachelabels(sheet);
       forceupdate(sheet);
-      return BROKENSHEET;
+      return _("Invalid record, loading aborted");
     }
     /*}}}*/
   }
@@ -1558,11 +1635,12 @@ const char *loadxdr(Sheet *sheet, const char *name)
   sheet->changed=0;
   cachelabels(sheet);
   forceupdate(sheet);
+  redraw_sheet(sheet);
   return (const char*)0;
 }
 /*}}}*/
 /* loadcsv       -- load/merge CSVs */ /*{{{*/
-const char *loadcsv(Sheet *sheet, const char *name, int semicolon)
+const char *loadcsv(Sheet *sheet, const char *name)
 {
   /* variables */ /*{{{*/
   FILE *fp;
@@ -1573,16 +1651,26 @@ const char *loadcsv(Sheet *sheet, const char *name, int semicolon)
   const char *str;
   double value;
   long lvalue;
+  int separator = 0;
   /*}}}*/
 
   if ((fp=fopen(name,"r"))==(FILE*)0) return strerror(errno);
   err=(const char*)0;
-  csv_setopt(semicolon);
   for (x=0,line=1; fgets(ln,sizeof(ln),fp); ++line)
   {
     const char *s;
     const char *cend;
 
+    if (!separator) { /* FIXME: find a better way to autodetect */
+        int ccnt = 0, scnt = 0;
+        char *pos = ln;
+        while ((pos = strchr(pos, ','))) pos++, ccnt++;
+        pos = ln;
+        while ((pos = strchr(pos, ';'))) pos++, scnt++;
+        if (ccnt || scnt) separator = 1;
+        csv_setopt(scnt > ccnt);
+    }
+
     s=cend=ln;
     x=0;
     do
@@ -1594,42 +1682,42 @@ const char *loadcsv(Sheet *sheet, const char *name, int semicolon)
       if (s!=cend) /* ok, it is a integer */ /*{{{*/
       {
         t[0]->type=INT;
-	t[0]->u.integer=lvalue;
-	putcont(sheet, sheet->curx+x, sheet->cury+line-1, sheet->curz, t, 0);
+        t[0]->u.integer=lvalue;
+        putcont(sheet, sheet->curx+x, sheet->cury+line-1, sheet->curz, t, 0);
       }
       /*}}}*/
       else
       {
-	value=csv_double(s,&cend);
-	if (s!=cend) /* ok, it is a double */ /*{{{*/
-	{
-	  t[0]->type=FLOAT;
-	  t[0]->u.flt=value;
-	  putcont(sheet, sheet->curx+x, sheet->cury+line-1, sheet->curz, t, 0);
-	}
-	/*}}}*/
-	else
-	{
-	  str=csv_string(s,&cend);
-	  if (s!=cend) /* ok, it is a string */ /*{{{*/
-	  {
-	    t[0]->type=STRING;
-	    t[0]->u.string=mystrmalloc(str);
-	    putcont(sheet, sheet->curx+x, sheet->cury+line-1, sheet->curz, t, 0);
-	  }
-	  /*}}}*/
-	  else
-	  {
-	    tvecfree(t);
-	    csv_separator(s,&cend);
-	    while (s==cend && *s && *s!='\n')
-	    {
-	      err="unknown values ignored";
-	      csv_separator(++s,&cend);
-	    }
-	    /* else it is nothing, which does not need to be stored :) */
-	  }
-	}
+        value=csv_double(s,&cend);
+        if (s!=cend) /* ok, it is a double */ /*{{{*/
+        {
+          t[0]->type=FLOAT;
+          t[0]->u.flt=value;
+          putcont(sheet, sheet->curx+x, sheet->cury+line-1, sheet->curz, t, 0);
+        }
+        /*}}}*/
+        else
+        {
+          str=csv_string(s,&cend);
+          if (s!=cend) /* ok, it is a string */ /*{{{*/
+          {
+            t[0]->type=STRING;
+            t[0]->u.string=mystrmalloc(str);
+            putcont(sheet, sheet->curx+x, sheet->cury+line-1, sheet->curz, t, 0);
+          }
+          /*}}}*/
+          else
+          {
+            tvecfree(t);
+            csv_separator(s,&cend);
+            while (s==cend && *s && *s!='\n')
+            {
+              err=_("unknown values ignored");
+              csv_separator(++s,&cend);
+            }
+            /* else it is nothing, which does not need to be stored :) */
+          }
+        }
       }
     } while (s!=cend ? s=cend,++x,1 : 0);
   }
@@ -1878,7 +1966,7 @@ const char *sortblock(Sheet *sheet, int x1, int y1, int z1, int x2, int y2, int
   cachelabels(sheet);
   forceupdate(sheet);
   sheet->changed=1;
-  if (norel) return UNCOMPAR;
+  if (norel) return _("uncomparable elements");
   else return (const char*)0;
 }
 /*}}}*/
